import multiprocessing
import time
import signal
import os

class TimeoutException(Exception):
    """
    It took too long to compile and execute. 
    """


class RunnableProcessing(multiprocessing.Process):
    """ 
    Run a function in a child process.
    """
    def __init__(self, func, *args, **kwargs):
        self.queue = multiprocessing.Queue(maxsize=1)
        args = (func,) + args
        multiprocessing.Process.__init__(self, target=self.run_func, args=args, kwargs=kwargs)

    def run_func(self, func, *args, **kwargs):
        try:
            result = func(*args, **kwargs)
            self.queue.put((True, result))
        except Exception as e:
            self.queue.put((False, e))

    def done(self):
        return self.queue.full()

    def result(self):
        return self.queue.get()


def timeout(seconds, force_kill=True):
    """ 
    Timeout decorator using Python multiprocessing.
    """
    def wrapper(function):
        def inner(*args, **kwargs):
            now = time.time()
            proc = RunnableProcessing(function, *args, **kwargs)
            proc.daemon = True
            proc.start()
            proc.join(seconds)
            if proc.is_alive():
                if force_kill:
                    os.kill(proc.pid, signal.SIGKILL)
                runtime = time.time() - now
                raise TimeoutException('timed out after {0} seconds'.format(runtime))
            assert proc.done()
            success, result = proc.result()
            if success:
                return result
            else:
                raise result
        return inner
    return wrapper